import sys
import numpy as np
import matplotlib.pyplot as plt
from sympy import *
from enum import Enum
import json

from corrector import *
from generator import *

def raiseBadArgument():
    sys.stdout.write("./example.py <ex_type> [-generateNew] [-seed <seed>]\n")
    sys.stdout.write("     where <ex_type> can be 'fraction_simplification', 'first_degree_equation'\n")
    sys.stdout.write("     -generateNew and seed are optional\n")
    exit("Bad arguments\n")

def checkAnswer(exercise, str_answer):
    try:
        if exercise.is_equation:
            if "=" not in str_answer:
                return ["This is an equation, each step should contain a \"=\" symbol!"]
            
            eqal_index = str_answer.find("=")
            left_str = str_answer[0: eqal_index]
            right_str = str_answer[eqal_index+1: len(str_answer)]
            left_expr = sympify(left_str, evaluate=False)
            right_expr = sympify(right_str, evaluate=False)

            if exercise.unknowns[0] not in left_expr.free_symbols and exercise.unknowns[0] not in right_expr.free_symbols:
                return ["The unknown should appear in the answer!"]
            if exercise.unknowns[0] in left_expr.free_symbols and exercise.unknowns[0] in right_expr.free_symbols:
                return ["Exercise not fully solved..."]
            if left_expr != exercise.unknowns[0] and right_expr != exercise.unknowns[0]:
                return ["Exercise not fully solved..."]
            
            if left_expr == exercise.unknowns[0]:
                answer = right_expr
            else:
                answer = left_expr
        else:
            answer = sympify(str_answer, evaluate=False)
    except Exception as x:
        return ["Invalid mathematical expression :", x]
    free_symbols = answer.free_symbols
    for sym in free_symbols:
        if sym not in exercise.params and sym not in exercise.unknowns:
            return ["Reference to unknown symbol: " + str(sym)]
    
    if str(simplify(answer)) == str(simplify(exercise.answers[0])):
        if str(answer) == str(exercise.answers[0]):
            return ["Correct answer, well done!"]
        else:
            return ["Answer could be further simplified!"]

    answer = together(simplify(answer))
    if str([answer]) not in exercise.answers_feedbacks:
        return ["Wrong answer due to unknown error in the development"]
    
    return exercise.answers_feedbacks[str([answer])]

def printFeedbacks(feedbacks):
    for feedback in feedbacks:
        print(feedback, "\n")

class Mode(Enum):
    FRACTION_SIMPLIFICATION = 1
    FIRST_DEGREE_EQUATION = 2

OPTIONS = {}
OPTIONS["fraction_simplification"] = Mode.FRACTION_SIMPLIFICATION
OPTIONS["first_degree_equation"] = Mode.FIRST_DEGREE_EQUATION

if len(sys.argv) < 2 or len(sys.argv) > 5 or not sys.argv[1] in OPTIONS :
    raiseBadArgument()

args = sys.argv.copy()

mode = OPTIONS[args[1]]
args.pop(1)

seed = None
if "-seed" in args:
    index = args.index("-seed")
    if len(args) < index+2:
        raiseBadArgument()
    seed = args[index+1]

    args.pop(index+1)
    args.pop(index)

generate_ex = False
if "-generateNew" in sys.argv:
    generate_ex = True
    args.pop(args.index("-generateNew"))

if len(args) > 1:
    raiseBadArgument()

config_file = open('generatorConfig.json')
if mode == Mode.FRACTION_SIMPLIFICATION:
    corrector = FracSimpCorrector()
    generator = FracSimpGenerator(config_file, corrector, seed=seed)
    example_file_name = "fracSimpExample.json"
if mode == Mode.FIRST_DEGREE_EQUATION:
    corrector = FirstDegEquCorrector()
    generator = FirstDegEquGenerator(config_file, corrector, seed=seed)
    example_file_name = "firstDegEquExample.json"
config_file.close()

if generate_ex:
    print("Generating exercise...")
    ex_json = generator.generateEx()
    ex_dict = json.loads(ex_json)
    with open(example_file_name, 'w') as fp:
        json.dump(ex_dict, fp)
    print("Done")

example_file = open(example_file_name)
ex_dict = json.load(example_file)
example_file.close()
ex = Exercise.fromDict(ex_dict)

ex.printEx()
print("\nProvide your reasoning followed by your answer line by line. Enter 'ok' once you are done:")

reasoning = []
if ex.is_equation:
    step = str(ex.ex_expr[0]) + " = " + str(ex.ex_expr[1])
else:
    step = str(ex.ex_expr[0])
while step != "ok":
    reasoning.append(step)
    step = input()

answer = reasoning[-1]
feedbacks = checkAnswer(ex, answer)
printFeedbacks(feedbacks)

if feedbacks != ["Correct answer, well done!"] and feedbacks != ["Answer could be further simplified!"]:
    print("Would you like better feedback ? [y/n]")

    if input() == 'y':
        feedbacks = corrector.getFeedbacksFromReasoning(json.dumps(ex_dict), reasoning)
        printFeedbacks(feedbacks)