from sympy import *
import numpy as np
from reasoningTree import *
class Exercise:

    def __init__(self, answer_type="MCQ", params=np.array([]), is_equation=False, ex_expr=[sympify('0')], unknowns=[],
                 factors=[] , answers=[], prt_message=""):
        '''
        answer_type (string) = Format of exercise (MCQ, open question, ...)
        params (array_like(sympy.Symbol)) = Parameters included in the exercise
        is_equation (boolean) = Allows to distinguish equation-like exercises (with left and right hand sides), and others which only require one exprerssion
        ex_expr (array_like(sympy expression)) = Expression of the exercise or expressions of the left and right hand sides of the exercise if it is an equation
        unknowns (array_like(sympy.Symbol)) = Array containing unknowns of equation-like exercise
        factors (array_like(sympy expression)) = For factorization-like exercises, list of factors to be factorized
        answers (array_like(sympy expression)) = Array containing the solution(s) to the erxercise
        prt_message (string) = Message to introduce the exercise
        development_tree (DevelopmentTree) = Tree representing all the ways the exerise can be solved step by step including all possible errors
        answers_feedbacks (dict(string, array_like(string))) = Dictionary associating feedbacks to considered possible answers
        '''
        self.answer_type = answer_type
        self.params = params
        self.is_equation = is_equation
        self.ex_expr = ex_expr
        self.unknowns = unknowns
        self.factors = factors
        self.answers = answers
        self.prt_message = prt_message
        self.answers_feedbacks = {}

    def printEx(self):
        if self.is_equation:
            print(self.prt_message, self.ex_expr[0], " = ", self.ex_expr[1])
        else:
            print(self.prt_message, self.ex_expr[0])
            
        print("The answer(s) is: ")
        for i in range(len(self.answers)):
            if self.is_equation:
                print(self.unknowns[0], " = ", self.answers[i])
            else:
                print(self.answers[i])

    def asDict(self):
        exercise_dict = {}
        exercise_dict["answer_type"] = self.answer_type
        exercise_dict["params"] = []
        for param in self.params:
            exercise_dict["params"].append(str(param))
        exercise_dict["is_equation"] = self.is_equation
        exercise_dict["ex_expr"] = []
        for expr in self.ex_expr:
            exercise_dict["ex_expr"].append(str(expr))
        exercise_dict["unknowns"] = []
        for unknown in self.unknowns:
            exercise_dict["unknowns"].append(str(unknown))
        exercise_dict["factors"] = []
        for factor in self.factors:
            exercise_dict["factors"].append(str(factor))
        exercise_dict["answers"] = []
        for answer in self.answers:
            exercise_dict["answers"].append(str(answer))
        exercise_dict["prt_message"] = self.prt_message
        exercise_dict["answers_feedbacks"] = self.answers_feedbacks

        return exercise_dict

    def fromDict(ex_as_dict):
        params = []
        for param in ex_as_dict["params"]:
            params.append(symbols(param))
        ex_expr = []
        for expr in ex_as_dict["ex_expr"]:
            ex_expr.append(sympify(expr, evaluate=False))
        unknowns = []
        for unknown in ex_as_dict["unknowns"]:
            unknowns.append(symbols(unknown))
        factors = []
        for factor in ex_as_dict["factors"]:
            factors.append(sympify(factor))
        answers = []
        for answer in ex_as_dict["answers"]:
            answers.append(together(simplify(sympify(answer))))

        ex = Exercise(answer_type=ex_as_dict["answer_type"], params=params, is_equation=ex_as_dict["is_equation"],
                      ex_expr=ex_expr, unknowns=unknowns, factors=factors, answers=answers, prt_message=ex_as_dict["prt_message"])
        ex.answers_feedbacks = ex_as_dict["answers_feedbacks"]

        return ex